from django import forms
from django.conf import settings
from django.utils.translation import get_language_bidi

__all__ = ['ChosenWidgetMixin', 'ChosenSelect', 'ChosenSelectMultiple',
        'ChosenGroupSelect']


class ChosenWidgetMixin:

    class Media:
        js = ("%s%s?v=2" % (settings.STATIC_URL, "chosen/js/chosen.jquery.min.js"),
              "%s%s?v=4" % (settings.STATIC_URL, "chosen/js/chosen.jquery_ready.js"))
        css = {"all": ("%s%s?v=1" % (settings.STATIC_URL, "chosen/css/chosen.css"), )}

    def __init__(self, attrs={}, *args, **kwargs):

        attrs['data-placeholder'] = kwargs.pop('overlay', None)
        attrs['class'] = "class" in attrs and self.add_to_css_class(
            attrs['class'], 'chosen-select') or "chosen-select"
        if get_language_bidi():
            attrs['class'] = self.add_to_css_class(attrs['class'], 'chosen-rtl')
        super(ChosenWidgetMixin, self).__init__(attrs, *args, **kwargs)

    def render(self, *args, **kwargs):
        if not self.is_required:
            self.attrs.update({'data-optional': True})
        return super(ChosenWidgetMixin, self).render(*args, **kwargs)

    def add_to_css_class(self, classes, new_class):
        new_classes = classes
        try:
            classes_test = u" " + str(classes) + u" "
            new_class_test = u" " + str(new_class) + u" "
            if new_class_test not in classes_test:
                new_classes += u" " + str(new_class)
        except TypeError:
            pass
        return new_classes


class ChosenSelect(ChosenWidgetMixin, forms.Select):
    pass


class ChosenSelectMultiple(ChosenWidgetMixin, forms.SelectMultiple):
    pass


class ChosenGroupSelect(ChosenSelect):

    def __init__(self, attrs={}, *args, **kwargs):
        super(ChosenGroupSelect, self).__init__(attrs, *args, **kwargs)
        attrs["class"] = "chosen-single chosen-with-drop"



from django import forms

from .widgets import ChosenSelect, ChosenSelectMultiple, ChosenGroupSelect

__all__ = [
    'ChosenFieldMixin', 'ChosenChoiceField', 'ChosenMultipleChoiceField',
    'ChosenModelChoiceField', 'ChosenModelMultipleChoiceField',
    'ChosenGroupChoiceField',
]


class ChosenFieldMixin(object):

    def __init__(self, *args, **kwargs):
        widget_kwargs = "overlay" in kwargs and\
            {"overlay": kwargs.pop('overlay')} or {}
        kwargs['widget'] = self.widget(**widget_kwargs)
        super(ChosenFieldMixin, self).__init__(*args, **kwargs)


class ChosenChoiceField(ChosenFieldMixin, forms.ChoiceField):

    widget = ChosenSelect


class ChosenMultipleChoiceField(ChosenFieldMixin, forms.MultipleChoiceField):

    widget = ChosenSelectMultiple


class ChosenModelChoiceField(ChosenFieldMixin, forms.ModelChoiceField):

    widget = ChosenSelect


class ChosenModelMultipleChoiceField(ChosenFieldMixin,
        forms.ModelMultipleChoiceField):

    widget = ChosenSelectMultiple


class ChosenGroupChoiceField(ChosenFieldMixin, forms.ChoiceField):
    """
    This field generate a Single Select with Groups (optgroup support).
    To render it correctly, you need to give a choice with the group title and
    the list of (id, value) for the subtitles

    A good way to do that is to add a Manager, eg::


        class MyModelManager(models.Manager):
            "Add get_group_choices to MyModel"

            def get_group_choices(self):
                '''
                Will filter the model per name and return tuples (obj.id,
                obj.rule)
                '''
                choices = []
                for name in MyModel.objects.values_list("name").distinct():
                    name = name[0]
                    name_choices = tuple((obj.id, obj.rule) for obj in
                        MyModel.objects.filter(name=name))
                    choices.append((name, name_choices))
                return choices
    """
    widget = ChosenGroupSelect



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

import chosen

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

def get_reqs(*fns):
    lst = []
    for fn in fns:
        for package in open(os.path.join(CURRENT_DIR, fn)).readlines():
            package = package.strip()
            if not package:
                continue
            lst.append(package.strip())
    return lst

setup(
    name='django-chosen-mega',
    version=chosen.__version__,
    description='django FormFields using the Chosen javascript plugin for jQuery',
    author='Chris Spencer',
    author_email='chrisspen@gmail.com',
    url='https://github.com/chrisspen/django-chosen',
    packages=find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.6",
        'Framework :: Django',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=get_reqs('requirements-min-django.txt', 'requirements.txt'),
    tests_require=get_reqs('requirements-test.txt'),
)
